Struct VarTuple
struct VarTuple<A, B> { ... }
A sized type that can be converted to a VarTupleULE.
See the module for examples.
Fields
sized: Avariable: B
Implementations
impl<'a, A, B, V> ZeroFrom for VarTuple<A, B>
fn zero_from(other: &'a VarTupleULE<A, V>) -> Self
impl<A, B> Freeze for VarTuple<A, B>
impl<A, B> RefUnwindSafe for VarTuple<A, B>
impl<A, B> Send for VarTuple<A, B>
impl<A, B> StructuralPartialEq for VarTuple<A, B>
impl<A, B> Sync for VarTuple<A, B>
impl<A, B> Unpin for VarTuple<A, B>
impl<A, B> UnsafeUnpin for VarTuple<A, B>
impl<A, B> UnwindSafe for VarTuple<A, B>
impl<A, B, V> EncodeAsVarULE for VarTuple<A, B>
fn encode_var_ule_as_slices<R, impl FnOnce(&[&[u8]]) -> R: FnOnce(&[&[u8]]) -> R>(self: &Self, _: impl FnOnce(&[&[u8]]) -> R) -> Rfn encode_var_ule_len(self: &Self) -> usizefn encode_var_ule_write(self: &Self, dst: &mut [u8])
impl<A: $crate::clone::Clone, B: $crate::clone::Clone> Clone for VarTuple<A, B>
fn clone(self: &Self) -> VarTuple<A, B>
impl<A: $crate::cmp::Eq, B: $crate::cmp::Eq> Eq for VarTuple<A, B>
impl<A: $crate::cmp::Ord, B: $crate::cmp::Ord> Ord for VarTuple<A, B>
fn cmp(self: &Self, other: &VarTuple<A, B>) -> Ordering
impl<A: $crate::cmp::PartialEq, B: $crate::cmp::PartialEq> PartialEq for VarTuple<A, B>
fn eq(self: &Self, other: &VarTuple<A, B>) -> bool
impl<A: $crate::cmp::PartialOrd, B: $crate::cmp::PartialOrd> PartialOrd for VarTuple<A, B>
fn partial_cmp(self: &Self, other: &VarTuple<A, B>) -> Option<Ordering>
impl<A: $crate::fmt::Debug, B: $crate::fmt::Debug> Debug for VarTuple<A, B>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<T> Any for VarTuple<A, B>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for VarTuple<A, B>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for VarTuple<A, B>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for VarTuple<A, B>
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> ErasedDestructor for VarTuple<A, B>
impl<T> From for VarTuple<A, B>
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for VarTuple<A, B>
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for VarTuple<A, B>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for VarTuple<A, B>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for VarTuple<A, B>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>