Struct Index16

struct Index16

This is a VarZeroVecFormat that stores u16s in the index array, and a u16 for a length.

Will have a smaller data size, but it's more likely for larger arrays to be unrepresentable (and error on construction)

This is the default index size used by all VarZeroVec types.

Implementations

impl Clone for Index16

fn clone(self: &Self) -> Index16

impl Copy for Index16

impl Debug for Index16

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Eq for Index16

impl Freeze for Index16

impl Hash for Index16

fn hash<__H: $crate::hash::Hasher>(self: &Self, state: &mut __H)

impl Ord for Index16

fn cmp(self: &Self, other: &Index16) -> Ordering

impl PartialEq for Index16

fn eq(self: &Self, other: &Index16) -> bool

impl PartialOrd for Index16

fn partial_cmp(self: &Self, other: &Index16) -> Option<Ordering>

impl RefUnwindSafe for Index16

impl Send for Index16

impl StructuralPartialEq for Index16

impl Sync for Index16

impl Unpin for Index16

impl UnsafeUnpin for Index16

impl UnwindSafe for Index16

impl VarZeroVecFormat for Index16

impl<T> Any for Index16

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for Index16

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for Index16

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for Index16

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> ErasedDestructor for Index16

impl<T> From for Index16

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for Index16

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for Index16

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for Index16

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for Index16

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>