Trait EqULE
unsafe trait EqULE: AsULE
A type whose byte sequence equals the byte sequence of its ULE type on little-endian platforms.
This enables certain performance optimizations, such as
ZeroVec::try_from_slice.
Implementation safety
This trait is safe to implement if the type's ULE (as defined by impl AsULE for T)
has an equal byte sequence as the type itself on little-endian platforms; i.e., one where
*const T can be cast to a valid *const T::ULE.
Implementors
impl EqULE for u8impl EqULE for u16impl EqULE for f32impl EqULE for u128impl EqULE for ()impl EqULE for i64impl EqULE for NonZeroU8impl EqULE for u32impl EqULE for f64impl EqULE for i16impl<T: EqULE, N: usize> EqULE for [T; N]impl EqULE for i128impl EqULE for i8impl EqULE for u64impl EqULE for boolimpl EqULE for i32