Struct Index8
struct Index8
This is a VarZeroVecFormat that stores u8s in the index array, and a u8 for a length.
Will have a smaller data size, but it's extremely likely for larger arrays to be unrepresentable (and error on construction). Should probably be used for known-small arrays, where all but the last field are known-small.
Implementations
impl Clone for Index8
fn clone(self: &Self) -> Index8
impl Copy for Index8
impl Debug for Index8
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Eq for Index8
impl Freeze for Index8
impl Hash for Index8
fn hash<__H: $crate::hash::Hasher>(self: &Self, state: &mut __H)
impl Ord for Index8
fn cmp(self: &Self, other: &Index8) -> Ordering
impl PartialEq for Index8
fn eq(self: &Self, other: &Index8) -> bool
impl PartialOrd for Index8
fn partial_cmp(self: &Self, other: &Index8) -> Option<Ordering>
impl RefUnwindSafe for Index8
impl Send for Index8
impl StructuralPartialEq for Index8
impl Sync for Index8
impl Unpin for Index8
impl UnsafeUnpin for Index8
impl UnwindSafe for Index8
impl VarZeroVecFormat for Index8
impl<T> Any for Index8
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for Index8
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for Index8
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for Index8
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> ErasedDestructor for Index8
impl<T> From for Index8
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for Index8
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for Index8
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for Index8
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for Index8
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>